﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Web;

using System.Globalization;

namespace dhtmlxConnectors
{
    /// <summary>
    /// connectors XML response Writter
    /// </summary>
    public class dhtmlxXMLWriter : IdhtmlxWriter
    {

        // operations counter
        private int counter = 0;
        /// <summary>
        /// stores StringBuilder
        /// </summary>
        protected StringBuilder _sb = null;
        /// <summary>
        /// stores XmlWriter
        /// </summary>
        protected XmlWriter xWriter = null;
        /// <summary>
        /// getter for StringBuilder
        /// </summary>
        public StringBuilder StringBuilder
        {
            get { return _sb; }
                
        }
        private HttpResponse _response = null;
        /// <summary>
        /// gets instance of httpResponse
        /// </summary>
        public HttpResponse Response
        {
            get { return _response; }
        }
        private void check()
        {
            //flush xmlWriter to StringBuilder every 100 operations
            if (counter >= 100)
                xWriter.Flush();
        }
        private void update()
        {
            counter++;
            check();
        }
        /// <summary>
        /// creates instance of dhtmlxXMLWriter
        /// </summary>
        /// <param name="builder">String builder</param>
        /// <param name="response">HttpResponse</param>
        public dhtmlxXMLWriter(StringBuilder builder, HttpResponse response)
        {
            this._sb = builder;
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.OmitXmlDeclaration = true;
            XmlWriter writer = XmlWriter.Create(builder, settings);
            this.xWriter = writer;
            update();
            _response = response;
        }
        /// <summary>
        /// writes start document tag
        /// </summary>
        /// <param name="title">start document tag</param>
        public void WriteStartDocument(string title)
        {
            xWriter.WriteStartElement(title);
            update();
        }
        /// <summary>
        /// writes start document tag
        /// </summary>
        public void WriteStartDocument()
        {
            xWriter.WriteStartElement("");
            update();
        }
        /// <summary>
        /// writes xml attribute
        /// </summary>
        /// <param name="name">attribute name</param>
        /// <param name="value">attribute value</param>
        public void WriteAttribute(string name, string value)
        {
            xWriter.WriteAttributeString(name, value != null ? _Preprocess(value) : "");
            update();
        }

        /// <summary>
        /// closes document tag
        /// </summary>
        public void WriteEndDocument()
        {

            xWriter.WriteEndElement();
            
        }
        /// <summary>
        /// writes name-value field, in this case simple xml node with text content
        /// </summary>
        /// <param name="name">node name</param>
        /// <param name="value">node value</param>
        public void WriteField(string name, string value)
        {
            xWriter.WriteStartElement(name);
            xWriter.WriteCData(value != null ? _Preprocess(value) : "");
            xWriter.WriteEndElement();
            update();
        }
        /// <summary>
        /// opens element tag
        /// </summary>
        /// <param name="title">element tag</param>
        public void WriteStartElement(string title)
        {
            xWriter.WriteStartElement(title);
            update();
        }
        /// <summary>
        /// opens element tag
        /// </summary>
    
        public void WriteStartElement()
        {
            xWriter.WriteStartElement("");
            update();
        }
        /// <summary>
        /// closes element tag
        /// </summary>
        public void WriteEndElement()
        {
            xWriter.WriteEndElement();
            update();
        }
        private static int fixed1 = int.Parse("9", NumberStyles.AllowHexSpecifier);
        private static int fixed2 = int.Parse("A", NumberStyles.AllowHexSpecifier);
        private static int fixed3 = int.Parse("D", NumberStyles.AllowHexSpecifier);
        private static int range1_1 = int.Parse("20", NumberStyles.AllowHexSpecifier);
        private static int range1_2 = int.Parse("D7FF", NumberStyles.AllowHexSpecifier);

        private static int range2_1 = int.Parse("E000", NumberStyles.AllowHexSpecifier);
        private static int range2_2 = int.Parse("FFFD", NumberStyles.AllowHexSpecifier);

        private static int range3_1 = int.Parse("10000", NumberStyles.AllowHexSpecifier);
        private static int range3_2 = int.Parse("10FFFF", NumberStyles.AllowHexSpecifier);


        private string _Preprocess(string value)
        {     
            var chars = value.ToCharArray();
            int code = 0;
            for(var i=0; i < chars.Length; i++){
                code =(int)chars[i];
                if(code != fixed1 && code != fixed2 && code != fixed3 &&
                   !(code >= range1_1 && code <= range1_2) &&
                   !(code >= range2_1 && code <= range2_2) &&
                   !(code >= range3_1 && code <= range3_2))
                  chars[i] = ' ';
            }
            return new string(chars);
        }

        /// <summary>
        /// writes string
        /// </summary>
        /// <param name="value">Calue</param>
        public void WriteString(string value)
        {
            xWriter.WriteCData(_Preprocess(value));
            update();
        }
        /// <summary>
        /// closes writer and returns result document
        /// </summary>
        /// <returns>string</returns>
        public string GetResult()
        {
            xWriter.Close();
            var result = this._sb.ToString();
            _sb.Remove(0, _sb.Length);
            return result;
        }

        /// <summary>
        /// writes string value to output(not necessary xml)
        /// </summary>
        /// <param name="value">string</param>
        public void WriteRawString(string value)
        {
            //xWriter.Flush();
            xWriter.WriteRaw(_Preprocess(value));
            update();
          //  _sb.Append(value);
        }
        /// <summary>
        /// cleares stringBuilder`s buffer
        /// </summary>
        /// <returns>string</returns>
        public void Reset()
        {
            _sb.Remove(0, _sb.Length);
        }
    }
}
